import requests

from requests_auth_mashery import MasheryAuth


class MasheryReportingApi(object):
    """
    This API enables customers to retrieve performance and operational data, for
    their Mashery-managed APIs, in order to drive off-line analytics, custom
    dashboards and integrate into their own datawarehouses. This API, which can
    be used with your existing Mashery V2 API key, exposes most of the data that
    can already be seen in the Mashery Dashboard.

    http://support.mashery.com/docs/read/mashery_api/20_reporting
    """

    BASE_URL = 'http://api.mashery.com/v2/rest'

    def __init__(self, site_id, service_key, api_key, api_secret):
        self.site_id = site_id
        self.service_key = service_key

        self.base_url = '%s/%s/reports/calls' % (self.BASE_URL, self.site_id)

        session = requests.Session()
        session.auth = MasheryAuth(api_key, api_secret)
        self.session = session

    def make_request(self, path, params=None):
        """
        Makes the API request to Mashery. Used by the other methods in
        this class.
        """

        if params is None:
            params = {}

        params['format'] = 'json'

        url = self.base_url + path

        r = self.session.get(url=url, params=params)

        r.raise_for_status()

        payload = r.json()

        if 'errorcode' in payload:
            raise Exception(payload['errorcode'])

        return payload

    def developer_activity_for_service(self, start_date, end_date, limit=10):
        """
        For a specific service, returns a summary across the report interval of
        activity by Developer. Sort order of the returned data is number of
        successful calls for each developer, in descending order.
        """

        path = '/developer_activity/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'limit': limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def errorcodes_for_service(self, start_date, end_date, errorcode_limit=10):
        """
        For a specific service, returns time-based counts of errors
        """

        path = '/errorcodes/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'errorcode_limit': errorcode_limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def errorcodes_for_service_for_developer(self, service_dev_key, start_date,
                                             end_date, errorcode_limit=10):
        """
        For a specific service, for a specific developer, call count by
        error code.
        """

        path = '/errorcodes/service/%(service_key)s/developer/%(service_dev_key)s' % {
            'service_key': self.service_key,
            'service_dev_key': service_dev_key
        }

        params = {
            'errorcode_limit': errorcode_limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def errorcount_by_developer_for_service(self, start_date, end_date,
                                            limit=10):
        """
        For a specific service, returns error count broken out by Developer
        """

        path = '/errorcount_by_developer/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'limit': limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def errorcount_by_method_for_service(self, start_date, end_date, limit=10):
        """
        For a specific service, returns time-based counts of errors further
        broken out by Method
        """

        path = '/errorcount_by_method/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'limit': limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def geolocation_for_service(self, start_date, end_date):
        """
        For a specific service, returns a summary across the report interval of
        originating geolocation of calls
        """

        path = '/geolocation/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def geolocation_for_service_for_developer(self, service_dev_key, start_date,
                                              end_date, limit=10):
        """
        For a specific service and developer, returns a summary across the
        report interval of originating geolocation of calls
        """

        path = '/geolocation/service/%(service_key)s/developer/%(service_dev_key)s' % {
            'service_key': self.service_key,
            'service_dev_key': service_dev_key
        }

        params = {
            'limit': limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def latency_for_service(self, start_date, end_date):
        """
        For a specific service, returns time-based average of call latency
        breaking out Mashery Proxy and the Client API
        """

        path = '/latency/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def latency_by_method_for_service(self, start_date, end_date,
                                      method_limit=10):
        """
        For a specific service, returns time-based average of call latency
        further broken out by Method
        """

        path = '/latency_by_method/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'method_limit': method_limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def latency_by_responsecode_for_service(self, start_date, end_date):
        """
        For a specific service, returns time-based average of call latency
        further broken out by response code
        """

        path = '/latency_by_responsecode/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def median_volume_by_hour_for_service(self, start_date, end_date):
        """
        For a specific service, across the specified interval, returns median
        call volume by hour for a 24-hour day.
        """

        path = '/median_volume_by_hour/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def methods_for_service(self, start_date, end_date, method_limit=10):
        """
        For a specific service, returns time-based count of calls by method
        """

        path = '/methods/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'method_limit': method_limit,
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def methods_for_service_for_developer(self, service_dev_key, start_date,
                                          end_date):
        """
        For a specific service, for a specific developer, call count by method.
        """

        path = '/methods/service/%(service_key)s/developer/%(service_dev_key)s' % {
            'service_key': self.service_key,
            'service_dev_key': service_dev_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def responses_from_cache_for_service(self, start_date, end_date):
        """
        For a specific service, returns time-based percentage of successful call
        responses that were from Mashery Cache
        """

        path = '/responses_from_cache/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def status_for_service(self, start_date, end_date):
        """
        For a specific service, returns time-based count of calls by status
        """

        path = '/status/service/%(service_key)s' % {
            'service_key': self.service_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)

    def status_for_service_for_developer(self, service_dev_key, start_date,
                                         end_date):
        """
        Call status for a specific service and a specific developer.
        """

        path = '/status/service/%(service_key)s/developer/%(service_dev_key)s' % {
            'service_key': self.service_key,
            'service_dev_key': service_dev_key
        }

        params = {
            'start_date': start_date,
            'end_date': end_date,
        }

        return self.make_request(path, params)



from setuptools import setup, find_packages


with open('README.rst') as fp:
    long_description = fp.read()


setup(
    name='masheryclient',
    version='0.1.0',

    description='A simple Python client library for the Mashery API',
    long_description=long_description,

    author='Devin Sevilla',
    author_email='dasevilla@gmail.com',

    url='https://github.com/dasevilla/mashery-python',
    download_url='https://github.com/dasevilla/mashery-python/tarball/master',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],

    install_requires=[
        'requests>=1.2.0',
        'requests-auth-mashery',
    ],

    packages=find_packages(),
)



import json

import requests

from requests_auth_mashery import MasheryAuth


class MasheryObjectType(object):
    """
    A Member object stores contact information for API key owners and allows
    developer portal access.
    """
    MEMBER = 'member'

    """
    An Application is the name of the client system that a developer is building
    that consumes either Packages or Services.
    """
    APPLICATION = 'application'

    """
    A Key tracks access to your Service.
    """
    KEY = 'key'

    """
    A Package Key tracks access to your Package.
    """
    PACKAGE_KEY = 'package_key'

    """
    A group of Plans.
    """
    PACKAGE = 'package'

    """
    A specific set of rules that define access to Services.
    """
    PLAN = 'plan'

    """
    A Service object describes a proxy service.  It also contains one or
    more Endpoints.
    """
    SERVICE = 'service'

    """
    A Role groups members and controls access to portal features and content.
    """
    ROLE = 'role'

    """
    """
    DEVELOPER_CLASS = 'developer_class'


class MasheryJsonRpcApi(object):
    """
    Make JSON-RPC 2.0 request to the Mashery 2.0 API

    http://support.mashery.com/docs/read/mashery_api/20
    """

    MASHERY_API_BASE = 'http://api.mashery.com/v2/json-rpc'

    def __init__(self, site_id, service_key, api_key, api_secret):
        self.site_id = site_id
        self.service_key = service_key

        session = requests.Session()
        session.auth = MasheryAuth(api_key, api_secret)
        self.session = session

    def make_request(self, method, params=None):
        """
        Makes the JSON-RPC 2.0 API call and returns the unwrapped result.
        """
        if params is None:
            params = []

        payload = {
            'id': 1,
            'jsonrpc': '2.0',
            'method': method,
            'params': params,
        }

        headers = {
            'Content-Type': 'application/json',
            'Accept': 'text/plain',
        }

        url = '%s/%s' % (self.MASHERY_API_BASE, self.site_id)

        r = self.session.post(url=url, headers=headers, data=json.dumps(payload))
        r.raise_for_status()

        payload = r.json()['result']

        return payload

    def fetch(self, object_type, identifier):
        """
        Fetch an object by it's primary identifier. See the table below for
        which field ob the object to use.

        =========== ================== =========
        Object Type Primary Identifier JSON Type
        =========== ================== =========
        application id                 integer
        member      username           string
        key         id                 integer
        role        id                 integer
        service     service_key        string
        =========== ================== =========
        """
        method = '%s.fetch' % object_type
        params = [identifier]

        return self.make_request(method, params)

    def fetch_key_by_api_key(self, api_key):
        """
        Shortcut method for retrieving a Key object by `apikey` instead of by
        ID. It's more common that you'll just have the value of `apikey` field.
        """
        identifier = [{
            'service_key': self.service_key,
            'apikey': api_key,
        }]

        return self.fetch('key', identifier)

    def query(self, query):
        """
        Ad-hoc queries for objects.

        http://support.mashery.com/docs/read/mashery_api/20/Query_Language
        """
        return self.make_request('object.query', [query])
